<?php

/**
 * @file
 */

class pay_realex_method_direct_redirect extends pay_method_direct {
  var $pay_realex_merchant_id = '';
  var $pay_realex_account_id = 'internet';
  var $pay_realex_shared_secret = '';
  var $pay_realex_modify_order_id = '';
  var $auto_settle_flag = 1;
  var $pay_realex_url = 'https://epage.payandshop.com/epage.cgi';
  var $pay_realex_callback = '';

  /**
   * A list of all currencies supported by Realex.
   */
  function available_currencies() {
    return pay_realex_currencies();
  }

  function direct_request() {
    // Set the transaction type based on requested activity.
    if (!$txntype = $this->realex_trxtype($this->activity->activity)) {
      drupal_set_message(t('Payment activity "@activity" is unsupported.  Not processing transaction.', array('@activity' => $this->activity->activity)), 'error');
      return FALSE;
    }

    $host = $this->pay_realex_url;
    $merchantid = $this->pay_realex_merchant_id;
    $realex_accountid = $this->pay_realex_account_id;
    $secret = $this->pay_realex_shared_secret;

    // Create the timestamp in the format required by Realex Payments.
    $timestamp = strftime("%Y%m%d%H%M%S");
    mt_srand((double) microtime() * 1000000);

    // Realex takes the value in cent.
    $amount = round($this->total() * 100);
    $currency = $this->activity->currency();

    // Creates timestamp that is needed to make up orderid.
    // Get the transaction created time, if not set fall back to now.
    if (!empty($this->activity->pxid)) {
      $transaction = pay_load('pay_transaction', $this->activity->pxid);
      $time = $transaction->created;
      $timestamp = strftime('%Y%m%d%H%M%S', $time);
      $orderid = $timestamp . '-' . $this->activity->pxid;
    }
    else {
      $time = time();
      $timestamp = strftime('%Y%m%d%H%M%S', $time);
      mt_srand((double)microtime() * 1000000);
      $orderid = $timestamp . '-' . mt_rand(1, 999);
    }

    // Test whether subaccount should be added to Order ID.
    if (!empty($this->pay_realex_modify_order_id)) {
      $orderid .= '-' . $realex_accountid;
    }

    // Create the SHA1 hash.
    $tmp      = "$timestamp.$merchantid.$orderid.$amount.$currency";
    $sha1hash = sha1($tmp);
    $tmp      = "$sha1hash.$secret";
    $sha1hash = sha1($tmp);

    $data = array(
      'MERCHANT_ID' => $merchantid,
      'ACCOUNT' => $realex_accountid,
      'ORDER_ID' => $orderid,
      'CURRENCY' => $currency,
      'AMOUNT' => $amount,
      'TIMESTAMP' => $timestamp,
      'SHA1HASH' => $sha1hash,
      'VAR_REF' => check_plain($this->mail),
      'AUTO_SETTLE_FLAG' => $this->auto_settle_flag,
      'pxid' => $this->pxid,
      'paid' => $this->activity->paid,
      'ip_address' => $ip_address(),
    );

    // TODO - POST action?
    $req = '';
    foreach ($data as $key => $val) {
      if ($val = trim($val)) $req .= check_plain($key) . '=' . urlencode($val) . '&';
    }
    return $host . '?' . $req;
  }

  function direct_response($response = array()) {
    watchdog('pay_realex_redirect', t('paid: @paid, pxid: @pxid', array('@paid' => $_POST['paid'], '@pxid' => $_POST['pxid'])));
watchdog('pay_realex_redirect', 'request: ' . print_r($response, TRUE));

    $this->activity->result = FALSE;

    // Grab the fields Realex Payments POSTs back to this script after a card has
    // been authorised.
    $timestamp  = $_POST['TIMESTAMP'];
    $result     = $_POST['RESULT'];
    $orderid    = $_POST['ORDER_ID'];
    $message    = $_POST['MESSAGE'];
    $authcode   = $_POST['AUTHCODE'];
    $pasref     = $_POST['PASREF'];
    $realexsha1 = $_POST['SHA1HASH'];
    $batch      = $_POST['BATCHID'];
    $pxid       = $_POST['pxid'];
    $paid       = $_POST['paid'];

    $merchantid = $this->pay_realex_merchant_id;
    $realex_accountid = $this->pay_realex_account_id;
    $secret = $this->pay_realex_shared_secret;

    // This digital signature should correspond to the one Realex returns to this
    // script and can therefore be used to verify the message Realex sends back.
    $tmp      = "$timestamp.$merchantid.$orderid.$result.$message.$pasref.$authcode";
    $sha1hash = sha1($tmp);
    $tmp      = "$sha1hash.$secret";
    $sha1hash = sha1($tmp);

    // Check to see if hashes match or not.
    if ($sha1hash != $realexsha1) {
      watchdog('pay_realex', t('Hash match failed for order @order_id.', array('@order_id' => $_POST['ORDER_ID'])));
      drupal_set_message(t("Hashes don't match - response not authenticated!"));
    }

    $pay_activity = pay_load('pay_activity', $paid);

    // Success.
    if ($result == '00') {
      $this->activity->result = TRUE;
      $this->activity->action = 'complete';
      $this->activity->transaction_total = $this->total();

      $output .= '<h3>' . t('Payment Successful!') . '</h3>';
      $options = array();
      $options['absolute'] = TRUE;
      $output .= '<h2>' . l(t('Click to return to website'), '<front>', $options) . '</h2>';
    }

    // Failure.
    else {
      $this->activity->result = FALSE;

      $output .= '<h2>' . t('Payment failed') . '</h2>';
      $output .= '<h3>' . t('Reason provided:') . ' ' . filter_xss($message) . '</h3>';
      $options = array();
      $options['absolute'] = TRUE;
      $output .= '<h2>' . l(t('Click to return to website'), '<front>', $options) . '</h2>';
    }
    $this->activity->total = $this->total();
    $this->activity->identifier = $orderid;
    $this->activity->data[] = $response;

    print $output;
    return $this->activity->result;
  }

  function settings_form(&$form, &$form_state) {
    parent::settings_form($form, $form_state);
    $group = $this->handler();

    $form[$group]['realex_redirect']['#type'] = 'fieldset';
    $form[$group]['realex_redirect']['#collapsible'] = FALSE;
    $form[$group]['realex_redirect']['#title'] = t('Realex Redirect settings');
    $form[$group]['realex_redirect']['#group'] = $group;

    $form[$group]['realex_redirect']['pay_realex_merchant_id'] = array(
      '#type' => 'textfield',
      '#title' => t('Realex Merchant ID'),
      '#default_value' => $this->pay_realex_merchant_id,
      '#required' => TRUE,
      '#size' => 16,
      '#parents' => array($group, 'pay_realex_merchant_id'),
    );

    $form[$group]['realex_redirect']['pay_realex_account_id'] = array(
      '#type' => 'textfield',
      '#title' => t('Realex Account ID'),
      '#default_value' => $this->pay_realex_account_id,
      '#required' => TRUE,
      '#size' => 16,
      '#parents' => array($group, 'pay_realex_account_id'),
    );

    $form[$group]['realex_redirect']['pay_realex_shared_secret'] = array(
      '#type' => 'textfield',
      '#title' => t('Shared secret for order verification'),
      '#description' => t('Your Shared Secret as supplied by Realex.'),
      '#default_value' => $this->pay_realex_shared_secret,
      '#required' => TRUE,
      '#size' => 16,
      '#parents' => array($group, 'pay_realex_shared_secret'),
    );

    $form[$group]['realex_redirect']['pay_realex_modify_order_id'] = array(
      '#type' => 'checkbox',
      '#title' => t('Modify Order ID'),
      '#description' => t('Append your account ID to your Order ID. Useful if you have multiple sub accounts on your Realex Account.'),
      '#default_value' => $this->pay_realex_modify_order_id,
      '#parents' => array($group, 'pay_realex_modify_order_id'),
    );

    $form[$group]['realex_redirect']['pay_realex_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Realex Gateway URL'),
      '#description' => t('The Realex URL to submit the order to.'),
      '#default_value' => $this->pay_realex_url,
      '#required' => TRUE,
      '#size' => 32,
      '#parents' => array($group, 'pay_realex_url'),
    );

    $form[$group]['realex_redirect']['auto_settle_flag'] = array(
      '#type' => 'select',
      '#title' => t('Auto settle flag'),
      '#options' => array(1 => t('Enabled'), 0 => t('Disabled')),
      '#default_value' => $this->auto_settle_flag,
      '#required' => TRUE,
      '#parents' => array($group, 'auto_settle_flag'),
    );

    $form[$group]['realex_redirect']['pay_realex_callback'] = array(
      '#type' => 'textfield',
      '#title' => t('Realex Callback Page'),
      '#description' => t('This is the URL that Realex will redirect their response to. <br/> You should supply this to Realex.'),
      '#value' => url('pay/pay-realex-redirect/complete', array('absolute' => TRUE)),
      '#size' => 50,
      '#attributes' => array('readonly' => 'readonly'),
      '#parents' => array($group, 'pay_realex_callback'),
    );
  }

  function realex_trxtype($activity) {
    $trxtypes = array(
      'complete'  => 'Sale',
    );
    return $trxtypes[$activity];
  }

}
